@@ -30,9 +30,10 @@ typedef char** ARGV;
 ARGV XS_unpack_ARGV(SV *sv);
 void XS_pack_ARGV(SV *const sv, ARGV argv);
 
-struct capture *capture_new(SV *pn, ARGV argv, unsigned timeout, SV *cb[2], SV *input);
+struct capture *capture_new(SV *pn, ARGV argv, ARGV env, unsigned timeout, SV *cb[2], SV *input);
 void capture_DESTROY(struct capture *rc);
 char *capture_next_line(struct capture *rc, int fd);
 int capture_run(struct capture *cp);
 void capture_set_argv_ref(struct capture *cp, ARGV av);
+void capture_set_env_ref(struct capture *cp, ARGV av);
 void capture_set_input(struct capture *cp, SV *inp);
